Attribute VB_Name = "RTBEx"
Option Explicit
'
'Adds some extended functionality to the RichTextBox control on systems
'supporting version 2.0 RichEdit or later.
'
'Extensions provided here:
'
'   RTBReadUnicode
'       Read Unicode Text from the RTB.
'
'   RTBWriteUnicode
'       Write Unicode Text/SelText or TextRTF/SelRTF to the RTB.
'

Private Const CP_UNICODE = 1200&
Private Const GT_USECRLF = 1&
Private Const GTL_USECRLF = 1&
Private Const GTL_PRECISE = 2&
Private Const GTL_NUMCHARS = 8&

'Window message consts.
Private Const WM_USER = &H400&
Private Const EM_GETTEXTEX = WM_USER + 94
Private Const EM_GETTEXTLENGTHEX = WM_USER + 95
Private Const EM_SETTEXTEX = WM_USER + 97

Private Declare Function SendMessageWLng Lib "user32" Alias "SendMessageW" ( _
    ByVal hWnd As Long, _
    ByVal wMsg As Long, _
    ByVal wParam As Long, _
    ByVal lParam As Long) As Long
    
Public Enum RTBW_FLAGS
    'Flags for the SETEXTEX data structure.
    RTBW_DEFAULT = 0   'Deletes undo stack, discards RTF formatting, replaces all text.
    RTBW_KEEPUNDO = 1  'Keeps undo stack.
    RTBW_SELECTION = 2 'Replaces selection and keeps RTF formatting.
End Enum
'Force IDE to maintain the case of these constant names.
#If False Then
Dim RTBW_DEFAULT, RTBW_KEEPUNDO, RTBW_SELECTION
#End If

Private Type GETTEXTEX
    cb As Long
    flags As Long
    codepage As Long
    lpDefaultChar As Long
    lpUsedDefChar As Long
End Type

Private Type GETTEXTLENGTHEX
    flags As Long
    codepage As Long
End Type

Private Type SETTEXTEX
    flags As RTBW_FLAGS
    codepage As Long
End Type

Public Function RTBReadUnicode(ByVal RTB As RichTextLib.RichTextBox) As String
    'Reads Text from RichTextBox as Unicode text on a system with Rich Edit 3.0
    '(Windows Me, Windows 2000, or later).
    Dim gtlUnicode As GETTEXTLENGTHEX
    Dim gtUnicode As GETTEXTEX
    Dim lngChars As Long
    
    With gtlUnicode
        .flags = GTL_USECRLF Or GTL_PRECISE Or GTL_NUMCHARS
        .codepage = CP_UNICODE
    End With
    lngChars = SendMessageWLng(RTB.hWnd, EM_GETTEXTLENGTHEX, VarPtr(gtlUnicode), 0)
    
    With gtUnicode
        .cb = (lngChars + 1) * 2
        .flags = GT_USECRLF
        .codepage = CP_UNICODE
    End With
    RTBReadUnicode = String$(lngChars, 0)
    SendMessageWLng RTB.hWnd, EM_GETTEXTEX, VarPtr(gtUnicode), StrPtr(RTBReadUnicode)
End Function

Public Sub RTBWriteUnicode( _
    ByVal RTB As RichTextLib.RichTextBox, _
    ByRef Text As String, _
    Optional ByVal Scope As RTBW_FLAGS = RTBW_DEFAULT)
    'Writes Text to RichTextBox as Unicode text on a system with Rich Edit 3.0
    '(Windows Me, Windows 2000, or later).
    '
    'Caution/Feature:
    '   If Text begins with a valid RTF sequence such as "{\rtf" or "{urtf" the
    '   text is taken as RTF instead of plain text.
    Dim stUnicode As SETTEXTEX
    
    With stUnicode
        .flags = Scope
        .codepage = CP_UNICODE
    End With
    If SendMessageWLng(RTB.hWnd, EM_SETTEXTEX, VarPtr(stUnicode), StrPtr(Text)) = 0 Then
        Err.Raise &H8004AA00, "RTBWriteUnicode", "Failed: insufficient space available"
    End If
End Sub
